unit PlayerTr;

interface

uses
  Classes, windows, mmsystem, math;

type
  TSoundArray=array [0..0] of Byte;
  PSoundArray=^TSoundArray;

  TSoundPlayer = class(TThread)
  private
    { Private declarations }
  protected
    Header:array[0..2]of WAVEHDR;
    hEvent:THandle;
    procedure Execute; override;
    procedure Process;
    procedure ClosePlay;
    procedure WaveOutCheck(res:MMResult);
  end;

implementation

uses SoundMain;

procedure TSoundPlayer.WaveOutCheck(res:MMResult);
begin
 if Res=MMSYSERR_NOERROR then exit;
 ClosePlay;
end;

procedure TSoundPlayer.Execute;
begin
 Process;
end;

procedure TSoundPlayer.Process;
const
 DEFAULT_BLOCKSIZE=64*1024;
var
 BlockSize,ii,shift,size:Integer;
 si:TSystemInfo;
 curHdr:WAVEHDR;
 ofs:Int64;
 p:PSoundArray;
begin
 BlockSize:=DEFAULT_BLOCKSIZE;
 GetSystemInfo(si);
 ZeroMemory(@Header,sizeof(Header));

 hEvent:=CreateEvent(nil,false,false,nil);
 try
  WaveOutCheck(waveOutOpen(@SounderForm.w_hPlay, SounderForm.w_DeviceID, @SounderForm.wfx, hEvent, 0,CALLBACK_EVENT));

  Header[0].lpData:=VirtualAlloc(nil,round((BlockSize*2+Integer(si.dwPageSize)-1)/si.dwPageSize*si.dwPageSize),MEM_RESERVE+MEM_COMMIT,PAGE_READWRITE);
  if Header[0].lpData=nil then exit;
  Header[1].lpData:=Header[0].lpData+BlockSize;
  Header[0].dwBufferLength:=BlockSize;
  Header[1].dwBufferLength:=BlockSize;
  waveOutCheck(waveOutPrepareHeader(SounderForm.w_hPlay,@Header[0],sizeof(waveHDR)));
  waveOutCheck(waveOutPrepareHeader(SounderForm.w_hPlay,@Header[1],sizeof(waveHDR)));
  ii:=0;
  if SounderForm.DataSize<=BlockSize then ResetEvent(hEvent);
  while SounderForm.w_PlayPosition<SounderForm.DataSize do
   begin
    curHdr:=Header[ii and 1];
    if BlockSize>SounderForm.DataSize-SounderForm.w_PlayPosition then
     BlockSize:=SounderForm.DataSize-SounderForm.w_PlayPosition;
    Inc(ii);

    curhdr.dwBufferLength:=BlockSize;
    ofs:=SounderForm.w_PlayPosition+SounderForm.DataOffset;
    shift:=(ofs mod si.dwAllocationGranularity);
    ofs:=ofs-shift;
    size:=(BlockSize+ofs+si.dwPageSize-1);
    size:=size-(size mod Integer(si.dwPageSize));
    if size>SounderForm.wFileSize-ofs then size:=SounderForm.wFileSize-ofs;
    p:=MapViewOfFile(SounderForm.w_hData,FILE_MAP_READ, ofs shr 32 and $FFFFFFFF, ofs and $FFFFFFFF, Size);

    try
     CopyMemory(curhdr.lpData, @p[shift], min(size-shift,BlockSize));
    finally
     UnMapViewOfFile(p);
    end;

    waveOutCheck(waveOutWrite(SounderForm.w_hplay,@curhdr,sizeof(waveHDR)));
    if (WAIT_TIMEOUT=WaitForSingleObject(hEvent,10000)) then
     begin
      if not SounderForm.w_Pause then
       begin
        MessageBox(SounderForm.Handle,' ','',MB_OK+MB_ICONINFORMATION);
        break;
       end
      else
       WaitForSingleObject(hEvent,INFINITE);
     end;

    if SounderForm.w_Stop then ClosePlay;
    if SounderForm.w_Restart then
     begin
      waveOutReset(SounderForm.w_hPlay);
      SounderForm.w_Restart:=false;
      SounderForm.w_PlayPosition:=SounderForm.w_RestartPosition;
      BlockSize:=DEFAULT_BLOCKSIZE;
      if SounderForm.DataSize>BlockSize then SetEvent(hEvent);
      continue;
     end;
    SounderForm.w_PlayPosition:=SounderForm.w_PlayPosition+BlockSize;
   end;
 finally
  WaitForSingleObject(hEvent,500);
  ClosePlay;
 end;
end;

procedure TSoundPlayer.ClosePlay;
begin
 waveOutReset(SounderForm.w_hPlay);
 waveOutUnprepareHeader(SounderForm.w_hPlay, @Header[0], sizeof(waveHDR));
 waveOutUnprepareHeader(SounderForm.w_hPlay, @Header[1], sizeof(waveHDR));
 if Header[0].lpData<>nil then VirtualFree(Header[0].lpData,0,MEM_RELEASE);
 waveOutClose(SounderForm.w_hPlay);
 SounderForm.w_hPlay:=0;
 CloseHandle(hEvent);
 SounderForm.SoundTimer.Enabled:=false;
 SounderForm.PositionBar.Position:=0;
 Terminate;
 Free;
end;

end.
